@@ -101,6 +101,25 @@ class FakePlugin(object):
         LOG.debug("release_dhcp_port: %s %s", network_id, device_id)
 
 
+def empty_network():
+    """Construct and return an empty network model."""
+    return make_net_model({"id": NETWORK_ID,
+                           "subnets": [],
+                           "ports": [],
+                           "mtu": constants.DEFAULT_NETWORK_MTU})
+
+
+def make_net_model(net_spec):
+    try:
+        net_model = dhcp.NetModel(False, net_spec)
+    except TypeError:
+        # use_namespace option was removed during Mitaka cycle.
+        net_model = dhcp.NetModel(net_spec)
+        net_model._ns_name = None
+
+    return net_model
+
+
 class CalicoEtcdWatcher(EtcdWatcher):
 
     NETWORK_ID = 'calico'
@@ -117,14 +136,6 @@ class CalicoEtcdWatcher(EtcdWatcher):
     with ID 'calico', and many subnets within that network.
     """
 
-    def _empty_network(self):
-        """Construct and return an empty network model."""
-        return dhcp.NetModel(False,
-                             {"id": NETWORK_ID,
-                              "subnets": [],
-                              "ports": [],
-                              "mtu": constants.DEFAULT_NETWORK_MTU})
-
     def __init__(self, agent):
         super(CalicoEtcdWatcher, self).__init__(
             '127.0.0.1:4001',
@@ -134,7 +145,7 @@ class CalicoEtcdWatcher(EtcdWatcher):
         self.suppress_on_ports_changed = False
 
         # Create empty Calico network object in the cache.
-        self.agent.cache.put(self._empty_network())
+        self.agent.cache.put(empty_network())
 
         # Register the etcd paths that we need to watch.
         self.register_path(
@@ -281,12 +292,11 @@ class CalicoEtcdWatcher(EtcdWatcher):
             self.agent.call_driver('reload_allocations', net)
         else:
             # Subnets changed, so need to 'restart' the DHCP driver.
-            net = dhcp.NetModel(False,
-                                {"id": net.id,
-                                 "subnets": new_subnets,
-                                 "ports": net.ports,
-                                 "tenant_id": "calico",
-                                 "mtu": constants.DEFAULT_NETWORK_MTU})
+            net = make_net_model({"id": net.id,
+                                  "subnets": new_subnets,
+                                  "ports": net.ports,
+                                  "tenant_id": "calico",
+                                  "mtu": constants.DEFAULT_NETWORK_MTU})
             LOG.debug("new net: %s %s %s", net.id, net.subnets, net.ports)
 
             # Next line - i.e. just discarding the existing cache - is to work
@@ -383,7 +393,7 @@ class CalicoEtcdWatcher(EtcdWatcher):
         # Reset the cache.
         LOG.debug("Reset cache for new snapshot")
         self.agent.cache = NetworkCache()
-        self.agent.cache.put(self._empty_network())
+        self.agent.cache.put(empty_network())
 
         # Suppress the processing inside on_ports_changed, until we've
         # processed the whole snapshot.
@@ -462,7 +472,17 @@ class CalicoDhcpAgent(DhcpAgent):
 
         # Override settings that Calico's DHCP agent use requires.
         self.conf.set_override('enable_isolated_metadata', False)
-        self.conf.set_override('use_namespaces', False)
+        try:
+            self.conf.set_override('use_namespaces', False)
+        except cfg.NoSuchOptError:
+            # Option removed in Mitaka, with the behaviour of the reference
+            # DHCP agent code being hardcoded to assume that it should always
+            # _use_ a namespace - which unfortunately is wrong for Calico
+            # usage.  We compensate for this through a combination of the
+            # _make_net_model code above, and subclassing and overriding
+            # particular DHCP agent methods so that they don't do
+            # namespace-specific actions.
+            pass
         self.conf.set_override(
             'interface_driver',
             'networking_calico.agent.linux.interface.RoutedInterfaceDriver'
@@ -486,9 +506,31 @@ class CalicoDhcpAgent(DhcpAgent):
         self.etcd.loop()
 
 
+def setup_logging():
+    config.setup_logging()
+
+    # logging is set up as it is done for neutron agent, so
+    # in order to log additionally to file we simply need to add
+    # file handler to root logger
+    root_logger = logging.getLogger()
+
+    # FIXME(aroma-x) - move logging settings to configuration
+    # file in future
+    log_file = '/var/log/calico/dhcp-agent.log'
+    log_format = ('%(asctime)s [%(thread)d] (%(levelname)s) '
+                  '%(module)s: %(message)s')
+    log_level = logging.DEBUG
+
+    fh = logging.FileHandler(log_file)
+    fh.setLevel(log_level)
+    fh.setFormatter(logging.Formatter(log_format))
+
+    root_logger.addHandler(fh)
+
+
 def main():
     register_options(cfg.CONF)
     common_config.init(sys.argv[1:])
-    config.setup_logging()
+    setup_logging()
     agent = CalicoDhcpAgent()
     agent.run()
