@@ -204,7 +204,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/config/InterfacePrefix': 'tap',
             '/calico/v1/config/EndpointReportingEnabled': True,
             '/calico/v1/Ready': True,
-            '/calico/v1/policy/profile/SGID-default/rules': {
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules': {
                 "outbound_rules": [{"dst_ports": ["1:65535"],
                                     "dst_net": "0.0.0.0/0",
                                     "ip_version": 4},
@@ -217,7 +217,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                                   {"dst_ports": ["1:65535"],
                                    "src_tag": "SGID-default",
                                    "ip_version": 6}]},
-            '/calico/v1/policy/profile/SGID-default/tags':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
             ["SGID-default"]
         })
 
@@ -243,9 +243,11 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-1/workload/openstack/instance-1/'
             'endpoint/DEADBEEF-1234-5678':
                 {"name": "tapDEADBEEF-12",
-                 "profile_ids": ["SGID-default"],
+                 "profile_ids": ["openstack-sg-SGID-default"],
                  "mac": "00:11:22:33:44:55",
                  "ipv4_gateway": "10.65.0.1",
+                 "ipv4_nat": [{'int_ip': '10.65.0.2',
+                               'ext_ip': u'192.168.0.1'}],
                  "ipv4_nets": ["10.65.0.2/32"],
                  "ipv4_subnet_ids": ["subnet-id-10.65.0--24"],
                  "state": "active",
@@ -254,7 +256,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-1/workload/openstack/instance-2/'
             'endpoint/FACEBEEF-1234-5678':
                 {"name": "tapFACEBEEF-12",
-                 "profile_ids": ["SGID-default"],
+                 "profile_ids": ["openstack-sg-SGID-default"],
                  "mac": "00:11:22:33:44:66",
                  "ipv4_gateway": "10.65.0.1",
                  "ipv4_nets": ["10.65.0.3/32"],
@@ -262,7 +264,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                  "state": "active",
                  "ipv6_subnet_ids": [],
                  "ipv6_nets": []},
-            '/calico/v1/policy/profile/SGID-default/rules':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules':
                 {"outbound_rules": [{"dst_ports": ["1:65535"],
                                      "dst_net": "0.0.0.0/0",
                                      "ip_version": 4},
@@ -275,7 +277,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                                    {"dst_ports": ["1:65535"],
                                     "src_tag": "SGID-default",
                                     "ip_version": 6}]},
-            '/calico/v1/policy/profile/SGID-default/tags':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
                 ["SGID-default"]
         }
         self.assertEtcdWrites(expected_writes)
@@ -291,7 +293,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
         context = mock.MagicMock()
         context._port = lib.port1
         context._plugin_context.session.query.return_value.filter_by.\
-            side_effect = self.ips_for_port
+            side_effect = self.port_query
         self.driver.delete_port_postcommit(context)
         self.assertEtcdWrites({})
         self.assertEtcdDeletes(set(['/calico/v1/host/felix-host-1/workload/'
@@ -312,15 +314,17 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-1/workload/openstack/instance-1/'
             'endpoint/DEADBEEF-1234-5678':
                 {"name": "tapDEADBEEF-12",
-                 "profile_ids": ["SGID-default"],
+                 "profile_ids": ["openstack-sg-SGID-default"],
                  "mac": "00:11:22:33:44:55",
                  "ipv4_gateway": "10.65.0.1",
+                 "ipv4_nat": [{'int_ip': '10.65.0.2',
+                               'ext_ip': u'192.168.0.1'}],
                  "ipv4_nets": ["10.65.0.2/32"],
                  "ipv4_subnet_ids": ["subnet-id-10.65.0--24"],
                  "state": "active",
                  "ipv6_subnet_ids": [],
                  "ipv6_nets": []},
-            '/calico/v1/policy/profile/SGID-default/rules':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules':
                 {"outbound_rules": [{"dst_ports": ["1:65535"],
                                      "dst_net": "0.0.0.0/0",
                                      "ip_version": 4},
@@ -333,7 +337,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                                    {"dst_ports": ["1:65535"],
                                     "src_tag": "SGID-default",
                                     "ip_version": 6}]},
-            '/calico/v1/policy/profile/SGID-default/tags':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
                 ["SGID-default"]
         }
         self.assertEtcdWrites(expected_writes)
@@ -350,9 +354,10 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
         expected_writes['/calico/v1/host/new-host/workload/openstack/'
                         'instance-1/endpoint/DEADBEEF-1234-5678'] = {
             "name": "tapDEADBEEF-12",
-            "profile_ids": ["SGID-default"],
+            "profile_ids": ["openstack-sg-SGID-default"],
             "mac": "00:11:22:33:44:55",
             "ipv4_gateway": "10.65.0.1",
+            "ipv4_nat": [{'int_ip': '10.65.0.2', 'ext_ip': u'192.168.0.1'}],
             "ipv4_nets": ["10.65.0.2/32"],
             "ipv4_subnet_ids": ["subnet-id-10.65.0--24"],
             "state": "active",
@@ -375,9 +380,11 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-1/workload/openstack/instance-1/'
             'endpoint/DEADBEEF-1234-5678':
                 {"name": "tapDEADBEEF-12",
-                 "profile_ids": ["SGID-default"],
+                 "profile_ids": ["openstack-sg-SGID-default"],
                  "mac": "00:11:22:33:44:55",
                  "ipv4_gateway": "10.65.0.1",
+                 "ipv4_nat": [{'int_ip': '10.65.0.2',
+                               'ext_ip': u'192.168.0.1'}],
                  "ipv4_nets": ["10.65.0.2/32"],
                  "ipv4_subnet_ids": ["subnet-id-10.65.0--24"],
                  "state": "active",
@@ -398,7 +405,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-2/workload/openstack/instance-3/'
             'endpoint/HELLO-1234-5678':
                 {"name": "tapHELLO-1234-",
-                 "profile_ids": ["SGID-default"],
+                 "profile_ids": ["openstack-sg-SGID-default"],
                  "mac": "00:11:22:33:44:66",
                  "ipv6_gateway": "2001:db8:a41:2::1",
                  "ipv6_subnet_ids": ["subnet-id-2001:db8:a41:2--64"],
@@ -406,7 +413,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                  "state": "active",
                  "ipv4_nets": [],
                  "ipv4_subnet_ids": []},
-            '/calico/v1/policy/profile/SGID-default/rules':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules':
                 {"outbound_rules": [{"dst_ports": ["1:65535"],
                                      "dst_net": "0.0.0.0/0",
                                      "ip_version": 4},
@@ -419,7 +426,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                                    {"dst_ports": ["1:65535"],
                                     "src_tag": "SGID-default",
                                     "ip_version": 6}]},
-            '/calico/v1/policy/profile/SGID-default/tags':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
                 ["SGID-default"]
         }
         self.assertEtcdWrites(expected_writes)
@@ -522,12 +529,12 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             'rule'
         )
         self.assertEtcdWrites({
-            '/calico/v1/policy/profile/SG-1/rules':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/rules':
                 {"outbound_rules": [],
                  "inbound_rules": [{"dst_ports": ["5060:5061"],
                                     "src_tag": "SGID-default",
                                     "ip_version": 4}]},
-            '/calico/v1/policy/profile/SG-1/tags':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/tags':
                 ["SG-1"]
         })
 
@@ -544,7 +551,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-2/workload/openstack/instance-3/'
             'endpoint/HELLO-1234-5678':
                 {"name": "tapHELLO-1234-",
-                 "profile_ids": ["SG-1"],
+                 "profile_ids": ["openstack-sg-SG-1"],
                  "mac": "00:11:22:33:44:66",
                  "ipv6_gateway": "2001:db8:a41:2::1",
                  "ipv6_subnet_ids": ["subnet-id-2001:db8:a41:2--64"],
@@ -552,12 +559,12 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                  "state": "active",
                  "ipv4_nets": [],
                  "ipv4_subnet_ids": []},
-            '/calico/v1/policy/profile/SG-1/rules':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/rules':
                 {"outbound_rules": [],
                  "inbound_rules": [{"dst_ports": ["5060:5061"],
                                     "src_tag": "SGID-default",
                                     "ip_version": 4}]},
-            '/calico/v1/policy/profile/SG-1/tags':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/tags':
                 ["SG-1"]
         }
         self.assertEtcdWrites(expected_writes)
@@ -607,12 +614,12 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
 
         # Expect an etcd write
         expected_writes = {
-            '/calico/v1/policy/profile/SG-1/rules':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/rules':
                 {"outbound_rules": [],
                  "inbound_rules": [{"dst_ports": [5060],
                                     "src_tag": "SGID-default",
                                     "ip_version": 4}]},
-            '/calico/v1/policy/profile/SG-1/tags':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/tags':
                 ["SG-1"]
         }
         self.assertEtcdWrites(expected_writes)
@@ -664,7 +671,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/host/felix-host-2/workload/openstack/instance-3/'
             'endpoint/HELLO-1234-5678':
                 {"name": "tapHELLO-1234-",
-                 "profile_ids": ["SG-1"],
+                 "profile_ids": ["openstack-sg-SG-1"],
                  "mac": "00:11:22:33:44:66",
                  "ipv6_subnet_ids": [],
                  "ipv6_nets": [],
@@ -672,12 +679,12 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                  "ipv4_gateway": "10.65.0.1",
                  "ipv4_subnet_ids": ["subnet-id-10.65.0--24"],
                  "ipv4_nets": ["10.65.0.188/32"]},
-            '/calico/v1/policy/profile/SG-1/rules':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/rules':
                 {"outbound_rules": [],
                  "inbound_rules": [{"dst_ports": [5070],
                                     "src_tag": "SGID-default",
                                     "ip_version": 4}]},
-            '/calico/v1/policy/profile/SG-1/tags':
+            '/calico/v1/policy/profile/openstack-sg-SG-1/tags':
                 ["SG-1"]
         }
         self.assertEtcdWrites(expected_writes)
@@ -731,7 +738,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/config/InterfacePrefix': 'tap',
             '/calico/v1/config/EndpointReportingEnabled': True,
             '/calico/v1/Ready': True,
-            '/calico/v1/policy/profile/SGID-default/rules': {
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules': {
                 "outbound_rules": [{"dst_ports": ["1:65535"],
                                     "dst_net": "0.0.0.0/0",
                                     "ip_version": 4},
@@ -744,7 +751,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                                   {"dst_ports": ["1:65535"],
                                    "src_tag": "SGID-default",
                                    "ip_version": 6}]},
-            '/calico/v1/policy/profile/SGID-default/tags':
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
             ["SGID-default"]
         })
 
@@ -817,7 +824,7 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
             '/calico/v1/Ready': True,
             '/calico/v1/config/EndpointReportingEnabled': True,
             '/calico/v1/config/InterfacePrefix': 'tap',
-            '/calico/v1/policy/profile/SGID-default/rules': {
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules': {
                 'inbound_rules': [{'dst_ports': ['1:65535'],
                                    'ip_version': 4,
                                    'src_tag': 'SGID-default'},
@@ -830,7 +837,9 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
                                    {'dst_net': '::/0',
                                     'dst_ports': ['1:65535'],
                                     'ip_version': 6}]},
-            '/calico/v1/policy/profile/SGID-default/tags': ['SGID-default']
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags': [
+                'SGID-default'
+            ]
         })
         self.assertEtcdDeletes(set())
 
@@ -1007,7 +1016,97 @@ class TestPluginEtcd(lib.Lib, unittest.TestCase):
         else:
             rules = {"outbound_rules": [etcd_rule],
                      "inbound_rules": []}
-        common.validate_rules("profile_id", rules)
+        common.validate_profile("profile_id", rules)
+
+    def test_profile_prefixing(self):
+        """Startup with existing profile data from another orchestrator."""
+        # Check that we don't delete the other orchestrator's profile data.
+        self.etcd_data = {
+            '/calico/v1/policy/profile/SGID-default/rules': {
+                "outbound_rules": [{"dst_ports": ["1:65535"],
+                                    "dst_net": "0.0.0.0/0",
+                                    "ip_version": 4},
+                                   {"dst_ports": ["1:65535"],
+                                    "dst_net": "::/0",
+                                    "ip_version": 6}],
+                "inbound_rules": [{"dst_ports": ["1:65535"],
+                                   "src_tag": "SGID-default",
+                                   "ip_version": 4},
+                                  {"dst_ports": ["1:65535"],
+                                   "src_tag": "SGID-default",
+                                   "ip_version": 6}]},
+            '/calico/v1/policy/profile/SGID-default/tags':
+            ["SGID-default"]
+        }
+        self.give_way()
+        self.simulated_time_advance(31)
+        self.assertEtcdWrites({
+            '/calico/v1/config/InterfacePrefix': 'tap',
+            '/calico/v1/config/EndpointReportingEnabled': True,
+            '/calico/v1/Ready': True,
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules': {
+                "outbound_rules": [{"dst_ports": ["1:65535"],
+                                    "dst_net": "0.0.0.0/0",
+                                    "ip_version": 4},
+                                   {"dst_ports": ["1:65535"],
+                                    "dst_net": "::/0",
+                                    "ip_version": 6}],
+                "inbound_rules": [{"dst_ports": ["1:65535"],
+                                   "src_tag": "SGID-default",
+                                   "ip_version": 4},
+                                  {"dst_ports": ["1:65535"],
+                                   "src_tag": "SGID-default",
+                                   "ip_version": 6}]},
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
+            ["SGID-default"]
+        })
+        self.assertEtcdDeletes(set())
+
+    def test_old_openstack_data(self):
+        """Startup with existing but old OpenStack profile data."""
+        # Check that we clean it up.
+        self.etcd_data = {
+            '/calico/v1/policy/profile/openstack-sg-OLD/rules': {
+                "outbound_rules": [{"dst_ports": ["1:65535"],
+                                    "dst_net": "0.0.0.0/0",
+                                    "ip_version": 4},
+                                   {"dst_ports": ["1:65535"],
+                                    "dst_net": "::/0",
+                                    "ip_version": 6}],
+                "inbound_rules": [{"dst_ports": ["1:65535"],
+                                   "src_tag": "OLD",
+                                   "ip_version": 4},
+                                  {"dst_ports": ["1:65535"],
+                                   "src_tag": "OLD",
+                                   "ip_version": 6}]},
+            '/calico/v1/policy/profile/openstack-sg-OLD/tags': ["OLD"]
+        }
+        self.give_way()
+        self.simulated_time_advance(31)
+        self.assertEtcdWrites({
+            '/calico/v1/config/InterfacePrefix': 'tap',
+            '/calico/v1/config/EndpointReportingEnabled': True,
+            '/calico/v1/Ready': True,
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/rules': {
+                "outbound_rules": [{"dst_ports": ["1:65535"],
+                                    "dst_net": "0.0.0.0/0",
+                                    "ip_version": 4},
+                                   {"dst_ports": ["1:65535"],
+                                    "dst_net": "::/0",
+                                    "ip_version": 6}],
+                "inbound_rules": [{"dst_ports": ["1:65535"],
+                                   "src_tag": "SGID-default",
+                                   "ip_version": 4},
+                                  {"dst_ports": ["1:65535"],
+                                   "src_tag": "SGID-default",
+                                   "ip_version": 6}]},
+            '/calico/v1/policy/profile/openstack-sg-SGID-default/tags':
+            ["SGID-default"]
+        })
+        self.assertEtcdDeletes(set([
+            '/calico/v1/policy/profile/openstack-sg-OLD/tags',
+            '/calico/v1/policy/profile/openstack-sg-OLD/rules'
+        ]))
 
 
 class TestDriverStatusReporting(lib.Lib, unittest.TestCase):
@@ -1368,7 +1467,8 @@ class TestCalicoEtcdWatcher(unittest.TestCase):
                                  "openstack/wlid/endpoint/ep1"
         m_port_status_node.value = '{"status": "up"}'
         self.watcher._on_ep_set(m_port_status_node, "hostname", "wlid", "ep1")
-        ep_id = datamodel_v1.EndpointId("hostname", "openstack", "wlid", "ep1")
+        ep_id = datamodel_v1.WloadEndpointId("hostname", "openstack",
+                                             "wlid", "ep1")
         self.assertEqual(self.watcher._endpoints_by_host,
                          {"hostname": set([ep_id])})
         return m_port_status_node
